"""IP trunk product block that has all parameters of a subscription throughout its lifecycle."""

import ipaddress
from typing import Optional, TypeVar

from orchestrator.domain.base import ProductBlockModel
from orchestrator.forms.validators import UniqueConstrainedList
from orchestrator.types import SubscriptionLifecycle, strEnum
from pydantic import Field

from gso.products.product_blocks.router import RouterBlock, RouterBlockInactive, RouterBlockProvisioning


class IptrunkType(strEnum):
    DARK_FIBER = "Dark_fiber"
    LEASED = "Leased"


T = TypeVar("T", covariant=True)


class IptrunkSides(UniqueConstrainedList[T]):  # type: ignore[type-var]
    min_items = 2
    max_items = 2


class IptrunkSideBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="IptrunkSideBlock"
):
    iptrunk_side_node: RouterBlockInactive
    iptrunk_side_ae_iface: Optional[str] = None
    iptrunk_side_ae_geant_a_sid: Optional[str] = None
    iptrunk_side_ae_members: list[str] = Field(default_factory=list)
    iptrunk_side_ae_members_description: list[str] = Field(default_factory=list)


class IptrunkSideBlockProvisioning(IptrunkSideBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    iptrunk_side_node: RouterBlockProvisioning
    iptrunk_side_ae_iface: Optional[str] = None
    iptrunk_side_ae_geant_a_sid: Optional[str] = None
    iptrunk_side_ae_members: list[str] = Field(default_factory=list)
    iptrunk_side_ae_members_description: list[str] = Field(default_factory=list)


class IptrunkSideBlock(IptrunkSideBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    iptrunk_side_node: RouterBlock
    iptrunk_side_ae_iface: Optional[str] = None
    iptrunk_side_ae_geant_a_sid: Optional[str] = None
    iptrunk_side_ae_members: list[str] = Field(default_factory=list)
    iptrunk_side_ae_members_description: list[str] = Field(default_factory=list)


class IptrunkBlockInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="IptrunkBlock"
):
    """A trunk that's currently inactive, see {class}`IptrunkBlock`."""

    geant_s_sid: Optional[str] = None
    iptrunk_description: Optional[str] = None
    iptrunk_type: Optional[IptrunkType] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_isis_metric: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sides: IptrunkSides[IptrunkSideBlockInactive]


class IptrunkBlockProvisioning(IptrunkBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A trunk that's currently being provisioned, see {class}`IptrunkBlock`."""

    geant_s_sid: Optional[str] = None
    iptrunk_description: Optional[str] = None
    iptrunk_type: Optional[IptrunkType] = None
    iptrunk_speed: Optional[str] = None
    iptrunk_minimum_links: Optional[int] = None
    iptrunk_isis_metric: Optional[int] = None
    iptrunk_ipv4_network: Optional[ipaddress.IPv4Network] = None
    iptrunk_ipv6_network: Optional[ipaddress.IPv6Network] = None
    #
    iptrunk_sides: IptrunkSides[IptrunkSideBlockProvisioning]


class IptrunkBlock(IptrunkBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A trunk that's currently deployed in the network."""

    geant_s_sid: str
    """GÉANT service ID associated with this trunk. """
    iptrunk_description: str
    """A human-readable description of this trunk."""
    iptrunk_type: IptrunkType
    """The type of trunk, can be either dark fibre or leased capacity."""
    iptrunk_speed: str  # FIXME: should be of PhyPortCapacity type
    """The speed of the trunk, measured per interface associated with it."""
    iptrunk_minimum_links: int
    """The minimum amount of links the trunk should consist of."""
    iptrunk_isis_metric: int
    """The {term}`IS-IS` metric of this link"""
    iptrunk_ipv4_network: ipaddress.IPv4Network
    """The IPv4 network used for this trunk."""
    iptrunk_ipv6_network: ipaddress.IPv6Network
    """The IPv6 network used for this trunk."""
    iptrunk_sides: IptrunkSides[IptrunkSideBlock]
